"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
function mdxPostToMonths(posts, basePaths) {
    const months = posts
        .sort((a, b) => new Date(b.date).getTime() - new Date(a.date).getTime())
        .reduce((accum, { date }) => {
        const year = new Date(date).getFullYear().toString();
        const month = (new Date(date).getMonth() + 1).toString().padStart(2, '0');
        const monthId = `${year}/${month}`;
        const indexInAccum = accum.map((d) => d.id).indexOf(monthId);
        if (indexInAccum < 0) {
            return [
                ...accum,
                {
                    year,
                    month,
                    id: monthId,
                    basePath: `${basePaths.posts}/${year}/${month}`,
                    totalCount: posts.filter((post) => new Date(post.date).getFullYear().toString() === year &&
                        (new Date(post.date).getMonth() + 1)
                            .toString()
                            .padStart(2, '0') === month).length,
                },
            ];
        }
        return accum;
    }, []);
    return months.map(({ year, month, ...props }) => {
        const gte = `${year}-${month}`;
        const lt = month !== '12'
            ? `${year}-${(parseInt(month, 10) + 1).toString().padStart(2, '0')}`
            : `${(parseInt(year, 10) + 1).toString()}-01`;
        return {
            ...props,
            year,
            month,
            gte,
            lt,
        };
    });
}
function gatsbyCreateResolvers({ createResolvers }, themeOptions) {
    const { basePaths } = gatsby_theme_aoi_blog_utils_1.withDefaults(themeOptions);
    const resolvers = {
        Query: {
            allMdxPostMonths: {
                type: `[MdxPostMonth]!`,
                resolve: async (source, args, context, info) => {
                    const { entries } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                    });
                    return mdxPostToMonths(Array.from(entries), basePaths);
                },
            },
        },
        MdxPostGroupConnection: {
            slug: {
                type: `String!`,
                resolve: async (source, args, context, info) => gatsby_theme_aoi_blog_utils_1.fieldValueToSlug(source, basePaths),
            },
        },
        MdxPost: {
            categoriesSlug: {
                type: `[WithSlug]`,
                resolve: async (source, args, context, info) => source.categories?.map((name) => ({
                    name,
                    slug: gatsby_theme_aoi_blog_utils_1.createSlug(basePaths.category, name),
                })) ?? [],
            },
            tagsSlug: {
                type: `[WithSlug]`,
                resolve: async (source, args, context, info) => source.tags?.map((name) => ({
                    name,
                    slug: gatsby_theme_aoi_blog_utils_1.createSlug(basePaths.tag, name),
                })) ?? [],
            },
        },
    };
    createResolvers(resolvers);
}
exports.default = gatsbyCreateResolvers;
