"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const gatsby_core_utils_1 = require("gatsby-core-utils");
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
async function processMdxPostRelativeImage(source, context, type) {
    // Image is a relative path - find a corresponding file
    const mdxFileNode = context.nodeModel.findRootNodeAncestor(source, (node) => node.internal && node.internal.type === `File`);
    if (!mdxFileNode) {
        return undefined;
    }
    const imagePath = gatsby_core_utils_1.slash(path.join(mdxFileNode.dir, (source[type] ?? '')));
    const fileNode = await context.nodeModel.findOne({
        type: `File`,
        query: {
            filter: {
                absolutePath: {
                    eq: imagePath,
                },
            },
        },
    });
    return fileNode ?? undefined;
}
function mdxResolverPassthrough(fieldName) {
    return async (source, args, context, info) => {
        const type = info.schema.getType(`Mdx`);
        const mdxNode = context.nodeModel.getNodeById({
            id: source.parent,
        });
        const resolver = type?.getFields()[fieldName].resolve;
        if (!resolver)
            return {};
        const result = await resolver(mdxNode ?? {}, args, context, {
            fieldName,
        });
        return result;
    };
}
async function processAuthorRelativeImage(source, context, type) {
    const { dir } = source;
    const imagePath = gatsby_core_utils_1.slash(path.join(dir, (source[type] ?? '')));
    const fileNode = await context.nodeModel.findOne({
        type: `File`,
        query: {
            filter: {
                absolutePath: {
                    eq: imagePath,
                },
            },
        },
    });
    return fileNode ?? undefined;
}
function createSchemaCustomization({ actions, schema }, themeOptions) {
    const options = gatsby_theme_aoi_blog_utils_1.withDefaults(themeOptions);
    const { createTypes } = actions;
    createTypes(`
    type Social @dontInfer {
      name: String!
      url: String!
    }
    type AuthorMdxPosts @dontInfer {
      posts: [MdxPost]!
      totalCount: Int!
    }
    type Author implements Node @dontInfer {
      name: String!
      slug: String
      avatar: File
      description: String
      website: String
      websiteURL: String
      socials: [Social]
      posts: AuthorMdxPosts
    }
    type WithSlug @dontInfer {
      name: String!
      slug: String!
    }
    type MdxPostMonth {
      id: String!
      year: String!
      month: String!
      basePath: String!
      gte: String!
      lt: String!
      totalCount: Int!
    }
  `);
    createTypes(schema.buildObjectType({
        name: `Author`,
        fields: {
            posts: {
                type: `AuthorMdxPosts`,
                resolve: async (source, args, context, info) => {
                    const { entries, totalCount } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                        query: {
                            filter: { author: { name: { eq: source.name } } },
                        },
                    });
                    return {
                        posts: entries,
                        totalCount: await totalCount(),
                    };
                },
            },
            avatar: {
                type: `File`,
                resolve: async (source, args, context, info) => {
                    if (!source.avatar)
                        return null;
                    if (source.image___NODE && gatsby_theme_aoi_blog_utils_1.isString(source.image___NODE)) {
                        return context.nodeModel.getNodeById({ id: source.image___NODE });
                    }
                    return processAuthorRelativeImage(source, context, `avatar`);
                },
            },
        },
    }));
    createTypes(schema.buildObjectType({
        name: `MdxPost`,
        fields: {
            id: { type: `ID!` },
            title: { type: `String!` },
            slug: { type: `String!` },
            date: { type: `Date!`, extensions: { dateformat: {} } },
            categories: { type: `[String]` },
            tags: { type: `[String]` },
            author: {
                type: `Author`,
                resolve: async (source, args, context, info) => {
                    const author = await context.nodeModel.findOne({
                        type: 'Author',
                        query: {
                            filter: { name: { eq: source.author } },
                        },
                    });
                    return (author ?? {
                        name: source.author ?? 'Unknown author',
                    });
                },
            },
            image: {
                type: `File`,
                resolve: async (source, args, context, info) => {
                    if (source.image___NODE && gatsby_theme_aoi_blog_utils_1.isString(source.image___NODE)) {
                        return context.nodeModel.getNodeById({ id: source.image___NODE });
                    }
                    return processMdxPostRelativeImage(source, context, `image`);
                },
            },
            imageAlt: { type: `String` },
            body: {
                type: `String!`,
                resolve: mdxResolverPassthrough(`body`),
            },
            excerpt: {
                type: `String!`,
                args: {
                    pruneLength: {
                        type: `Int`,
                        defaultValue: options.excerptLength,
                    },
                    truncate: {
                        type: `Boolean`,
                        defaultValue: true,
                    },
                },
                resolve: mdxResolverPassthrough(`excerpt`),
            },
        },
        interfaces: [`Node`],
        extensions: {
            infer: false,
        },
    }));
}
exports.default = createSchemaCustomization;
